/*
    (C) 1995-97 AROS - The Amiga Research OS
    $Id: assignpath.c,v 1.1.1.1 2005/03/15 15:54:40 laire Exp $

    Desc: Create a non-binding (path) assign.
    Lang: english
*/
#include <exec/memory.h>
#include <proto/exec.h>
#include <dos/dos.h>
#include <dos/dosextens.h>
#include "dos_intern.h"

/*****************************************************************************

    NAME */
#include <proto/dos.h>

	AROS_LH2(LONG, AssignPath,

/*  SYNOPSIS */
	AROS_LHA(CONST_STRPTR, name, D1),
	AROS_LHA(CONST_STRPTR, path, D2),

/*  LOCATION */
	struct DosLibrary *, DOSBase, 104, Dos)

/*  FUNCTION
	Create an assign for the given name, which will be resolved upon
	each reference to it. There will be no permanent lock kept on the
	specified path. This way you can create assigns to unmounted volumes
	which will only be requested when accessed. Also, using AssignPath()
	to assign C: to df0:c would make references go to to df0:c even if
	you change the disk.

    INPUTS
	name  -- NULL terminated name of the assign.
	path  -- NULL terminated path to be resolved on each reference.

    RESULT
	!= 0 in case of success, 0 on failure. IoErr() gives additional
	information in that case.

    NOTES

    EXAMPLE

    BUGS

    SEE ALSO
	AssignAdd(), AssignLock(), AssignLate(), Open()

    INTERNALS

    HISTORY
	27-11-96    digulla automatically created from
			    dos_lib.fd and clib/dos_protos.h

*****************************************************************************/
{
    AROS_LIBFUNC_INIT
    AROS_LIBBASE_EXT_DECL(struct DosLibrary *,DOSBase)

    CONST_STRPTR    s2;
    struct DosList *dl, *newdl;

    LONG    result = DOSTRUE;
    STRPTR  pathcopy;
    ULONG   namelen;

    DEBUG_ASSIGN(dprintf("AssignPath: name = \"%s\", path = \"%s\"\n", name, path));

    FAILFORTASKNP(return DOSFALSE;)

    newdl = MakeDosEntry(name, DLT_NONBINDING);

    if (newdl == NULL)
    {
	DEBUG_ASSIGN(dprintf("AssignPath: fail (no dos entry)\n"));
	return DOSFALSE;
    }

    s2 = path;

    while(*s2++)
	;
    
    namelen = s2 - path;
    pathcopy = dosAllocVec(namelen, MEMF_PUBLIC | MEMF_CLEAR);

    if (pathcopy == NULL)
    {
	DEBUG_ASSIGN(dprintf("AssignPath: fail (no mem)\n"));

	FreeDosEntry(newdl);
	SetIoErr(ERROR_NO_FREE_STORE);

	return DOSFALSE;
    }

    CopyMem((STRPTR) path, pathcopy, namelen);
    newdl->dol_misc.dol_assign.dol_AssignName = pathcopy;

    dl = LockDosList(LDF_ALL | LDF_WRITE);
    dl = FindDosEntry(dl, name, LDF_ALL);

    if (dl == NULL)
    {
	AddDosEntry(newdl);
    }
    else if (dl->dol_Type == DLT_VOLUME || dl->dol_Type == DLT_DEVICE)
    {
	DEBUG_ASSIGN(dprintf("AssignPath: fail (exists)\n"));

	dl = NULL;
	dosFreeVec(newdl->dol_misc.dol_assign.dol_AssignName);
	FreeDosEntry(newdl);
	SetIoErr(ERROR_OBJECT_EXISTS);

	result = DOSFALSE;
    }
    else
    {
	RemDosEntry(dl);
	AddDosEntry(newdl);
    }
    
    if (dl != NULL)
    {
	UnLock(dl->dol_Lock);

	if (dl->dol_misc.dol_assign.dol_List != NULL)
	{
	    struct AssignList *al, *oal;

	    for(al = dl->dol_misc.dol_assign.dol_List; al; )
	    {
		UnLock(al->al_Lock);
		oal = al;
		al = al->al_Next;
		dosFreeVec(oal);
	    }
	}

	dosFreeVec(dl->dol_misc.dol_assign.dol_AssignName);
	FreeDosEntry(dl);
    }
    
    UnLockDosList(LDF_ALL | LDF_WRITE);

    return result;

    AROS_LIBFUNC_EXIT
} /* AssignPath */
